#!/bin/bash

source install_library.bash

echo "Installing kernel debs."
sudo dpkg -i *.deb || exit 1

# The automated kernel entries only work for grub2.
echo "Checking bootloader"
if [ -e /boot/grub/menu.lst ]; then
    KVERSION=`grep '^kernel.*vmlinuz' /boot/grub/menu.lst | \
	sed 's/.*vmlinuz-\([^\ ]*\).*/\1/g' | head -n 1`
    echo
    echo "It looks like your system uses grub as a bootloader, rather than "
    echo "grub2 (was this system upgraded from old Ubuntu?).  You will have "
    echo "to modify /boot/grub/menu.lst by hand to boot the new kernel."
    echo
    echo "Entries for your kernel will have been added to menu.lst.  But "
    echo "they should be modified as follows:"
    echo
    echo "1. Move the two stanzas out of the \"AUTOMAGIC KERNEL LIST\";"
    echo "   You can put them just above the \"### BEGIN AUTOMAGIC...\" line."
    echo
    echo "2. In the \"kernel\" line, add the argument \"BIGPHYSAREA=8192\"."
    echo
    echo "3. In some cases you will require an \"initramfs\" to boot the"
    echo "   kernel.  This can be created by running"
    echo
    echo "    \$ sudo mkinitramfs -o /boot/initrd.img-${KVERSION} ${KVERSION}"
    echo
    echo "   An \"initramfs\" line can then be added to the kernel stanzas."
    exit 1
fi

# Grep the grub entry for the system install...
function grub_entry {
    awk '($1 == "menuentry") {s+=1} (s==1) {print $0}' /boot/grub/grub.cfg
}

ROOT=`grub_entry | grep 'set root'`
INSMOD=`grub_entry | grep insmod`
BIGPHYS=8192
LINUXLINE=`grub_entry | grep /boot/vmlinuz`
SEARCHLINE=`grub_entry | grep 'search'`
KVERSION=`echo "$LINUXLINE" | sed 's/.*vmlinuz-\\([^\\ ]*\\).*/\\1/g'`
UUID=`echo "$SEARCHLINE" | sed 's/.*\ \([a-f0-9][a-f0-9\\\-]*\).*/\1/g'`
KROOT=`echo $LINUXLINE | sed 's/.*root=\([a-z0-9\/]*\).*/\1/g'`

# Build a sed substitution command
cmd=
for var in KVERSION UUID ROOT KROOT INSMOD BIGPHYS; do
    # Escape / in each variable... we have to double escape the \ here...
    text=`echo ${!var} | sed 's/\\//\\\\\\//g'`
    cmd="$cmd s/@${var}@/${text}/g;"
done

# Create 07_mce
cat 07_mce.in | sed "$cmd" | tee 07_mce

# Make initramfs; the grub entry assumes it exists.
echo "Creating initrd."
sudo mkinitramfs -o /boot/initrd.img-${KVERSION} ${KVERSION} || exit 1

# Install it and update boot menu
sudo install 07_mce /etc/grub.d/

echo "Generating boot menu, this may take a while.  I do not know why that is."
sudo update-grub || exit 1

echo
echo "Install complete.  Please inspect the grub configuration to confirm:"
echo " 1. that /etc/default/grub points to the correct default kernel entry."
echo " 2. that the kernel (vmlimuz-*) and initramfs (initrd.img-*)"
echo "    referred to in the bigphysarea kernel entry in /boot/grub/grub.cnf"
echo "    do indeed exist in /boot ."
